import { Component, inject } from '@angular/core';
import { FileEntry, FileLoaderService } from '../file-loader.service';
import { distinctUntilChanged, filter, forkJoin, from, Observable, of, tap } from 'rxjs';

import {
    CdkDragDrop,
    moveItemInArray,
    transferArrayItem,
} from '@angular/cdk/drag-drop';

@Component({
    selector: 'app-concatenator',
    template: `
        <button mat-button *ngIf="(filesAvailable$ | async)?.length === 0" (click)="loader.openDir()">Forrás mappa
            betöltése
        </button>
        <h2>Forrásfájlok</h2>
        <div
                cdkDropList
                #sourceFiles="cdkDropList"
                [cdkDropListConnectedTo]="[one, two, three]"
                [cdkDropListData]="filesAvailable$ | async"
                class="source-list"
                (cdkDropListDropped)="drop($event)">
            <div class="example-box" *ngFor="let item of filesAvailable$ | async" cdkDrag>{{item.name}}</div>
        </div>

        <div class="containers">
            <div class="target-container">
                <h2>Első oszlop</h2>
                <div
                        cdkDropList
                        #one="cdkDropList"
                        [cdkDropListData]="selectedOne"
                        [cdkDropListConnectedTo]="[sourceFiles]"
                        class="target-list"
                        (cdkDropListDropped)="drop($event)">
                    <div class="example-box" *ngIf="selectedOne.length === 0">Húzd ide a fájlt</div>
                    <div class="example-box" *ngFor="let item of selectedOne" cdkDrag>{{item.name}}</div>
                </div>
            </div>
            <div class="target-container">
                <h2>Második oszlop</h2>
                <div
                        cdkDropList
                        #two="cdkDropList"
                        [cdkDropListData]="selectedTwo"
                        [cdkDropListConnectedTo]="[sourceFiles]"
                        class="target-list"
                        (cdkDropListDropped)="drop($event)">
                    <div class="example-box" *ngIf="selectedTwo.length === 0">Húzd ide a fájlt</div>
                    <div class="example-box" *ngFor="let item of selectedTwo" cdkDrag>{{item.name}}</div>
                </div>
            </div>
            <div class="target-container">
                <h2>Harmadik oszlop</h2>
                <div
                        cdkDropList
                        #three="cdkDropList"
                        [cdkDropListData]="selectedThree"
                        [cdkDropListConnectedTo]="[sourceFiles]"
                        class="example-list"
                        (cdkDropListDropped)="drop($event)">
                    <div class="example-box" *ngIf="selectedThree.length === 0">Húzd ide a fájlt</div>
                    <div class="example-box" *ngFor="let item of selectedThree" cdkDrag>{{item.name}}</div>
                </div>
            </div>
        </div>

        <div class="terminal">
            <pre>{{generatedCode}}</pre>
        </div>
    `,
    styleUrls: ['concatenator.component.scss']
})
export class ConcatenatorComponent {
    loader = inject(FileLoaderService);

    selectedOne: FileEntry[] = [];
    selectedTwo: FileEntry[] = [];
    selectedThree: FileEntry[] = [];

    generatedCode: string = '';

    filesAvailable$: Observable<FileEntry[]> = this.loader.filesAvailable$.asObservable()
        .pipe(
            tap(files => console.log(files)),
            distinctUntilChanged((previous, current) => previous.length !== current.length)
        );

    constructor() {}

    drop(event: CdkDragDrop<FileEntry[] | any> | undefined) {
        if (!event || !event.container || !event.container.data) return;

        console.log(event);

        if (event.previousContainer === event.container) {
            return moveItemInArray(event.container.data, event.previousIndex, event.currentIndex);
        }
        transferArrayItem(
            event.previousContainer.data,
            event.container.data,
            event.previousIndex,
            event.currentIndex
        );

        this.updateGeneatedCode();
    }

    async updateGeneatedCode() {
        this.generatedCode = '';
        let data = [
            (this.selectedOne && this.selectedOne.length > 0 && this.isFile(this.selectedOne[0]?.handle) ? from(this.selectedOne[0].handle.getFile()) : of(null)),
            (this.selectedTwo && this.selectedTwo.length > 0 && this.isFile(this.selectedTwo[0]?.handle) ? from(this.selectedTwo[0].handle.getFile()) : of(null)),
            (this.selectedThree && this.selectedThree.length > 0 && this.isFile(this.selectedThree[0]?.handle) ? from(this.selectedThree[0].handle.getFile()) : of(null))
        ];

        forkJoin(
            data
        ).pipe(
            filter((file) => file !== null)
        ).subscribe((data) => {
            data.map((file) => {
                if (file) {
                    file.text().then((text) => {
                        this.generatedCode += text;
                    })
                }
            });
        })
    }

    private isFile(object: FileSystemHandle | FileSystemFileHandle): object is FileSystemFileHandle {
        return object?.kind === 'file'
    }

}
